function [A,Nu]=generateSynthDataFast(Z,Nlinks,Ngraphs)

% Input:
%  Z         parcellation given as region x N matrix
%  Nlinks    Number of links on average in the generated networks
%  Ntest     Number of test networks to generate
%
% Output:
%  A         cell array of Ngraphs graphs, each graph is an N x N
%            sparse matrix
%  Nu        Between cluster link densities used for generating graphs
%
% Written by Morten M?rup, June 2018

if nargin<3
    Ngraphs=1;
end

% Generate from uniform distribution link densities between groups
[noc,N]=size(Z);
Nu=betarnd(0.5*ones(noc),0.5*ones(noc));
%Z=sparse(z,1:length(z),ones(1,length(z)));
sumZ=sum(Z,2);
Entries=sumZ*sumZ'-Z*Z';
Entries=Entries-0.5*diag(diag(Entries));
P=full(Nu.*Entries);
P=P(triu(ones(noc))>0);
p=P(:)/sum(P(:));
r = mnrnd(Nlinks,p);
R=zeros(noc);
R(triu(ones(noc))>0)=r;

Itest=cell(1,Ngraphs);
Jtest=cell(1,Ngraphs);
for k=1:Ngraphs
    Itest{k}=nan(round(Nlinks*1.1),1);
    Jtest{k}=nan(round(Nlinks*1.1),1);
    i_start(k)=0;
end
   
for m=1:size(R,1)
    for l=m:size(R,1)      
        for k=1:Ngraphs
            if l==m
                %[Itmp,Jtmp]=find(triu(sprand(sumZ(m),sumZ(l),R(m,l)/(sumZ(m)*(sumZ(l)-1)/2)),1));        
                [Itmp,Jtmp]=find(triu(rand(sumZ(m),sumZ(l))<(R(m,l)/(sumZ(m)*(sumZ(l)-1)/2)),1));        
            else
                %[Itmp,Jtmp]=find(sprand(sumZ(m),sumZ(l),R(m,l)/(sumZ(m)*sumZ(l))));
                [Itmp,Jtmp]=find(rand(sumZ(m),sumZ(l))<(R(m,l)/(sumZ(m)*sumZ(l))));
            end                        
            Itest{k}((i_start(k)+1):(i_start(k)+length(Itmp)))=Itmp+sum(sumZ(1:m-1));
            Jtest{k}((i_start(k)+1):(i_start(k)+length(Itmp)))=Jtmp+sum(sumZ(1:l-1));
            i_start(k)=i_start(k)+length(Itmp);
        end
    end
end

[clust,dummy]=find(Z);
[val,perm]=sort(clust,'ascend');
Permutation=sparse(perm,1:N,ones(1,N));
for k=1:Ngraphs    
    A{k}=sparse(Itest{k}(1:i_start(k)),Jtest{k}(1:i_start(k)),ones(i_start(k),1),N,N);
    A{k}=triu(Permutation*(A{k}+A{k}')*Permutation',1);    
    
end